@@ -11,7 +11,7 @@ from typing import Union, Tuple
 import torch
 from torch.nn.modules.loss import _Loss
 
-from piq.utils import _adjust_dimensions, _validate_input
+from piq.utils import _validate_input, _reduce, _version_tuple
 from piq.functional import ifftshift, get_meshgrid, similarity_map, gradient_map, scharr_filter, rgb2yiq
 
 
@@ -21,13 +21,13 @@ def fsim(x: torch.Tensor, y: torch.Tensor, reduction: str = 'mean',
          mult: int = 2, sigma_f: float = 0.55, delta_theta: float = 1.2,
          k: float = 2.0) -> torch.Tensor:
     r"""Compute Feature Similarity Index Measure for a batch of images.
-    
 
     Args:
-        x: Predicted images. Shape (H, W), (C, H, W) or (N, C, H, W).
-        y: Target images. Shape (H, W), (C, H, W) or (N, C, H, W).
-        reduction: Reduction over samples in batch: "mean"|"sum"|"none"
-        data_range: Value range of input images (usually 1.0 or 255). Default: 1.0
+        x: An input tensor. Shape :math:`(N, C, H, W)`.
+        y: A target tensor. Shape :math:`(N, C, H, W)`.
+        reduction: Specifies the reduction type:
+            ``'none'`` | ``'mean'`` | ``'sum'``. Default:``'mean'``
+        data_range: Maximum value range of images (usually 1.0 or 255).
         chromatic: Flag to compute FSIMc, which also takes into account chromatic components
         scales: Number of wavelets used for computation of phase congruensy maps
         orientations: Number of filter orientations used for computation of phase congruensy maps
@@ -41,26 +41,30 @@ def fsim(x: torch.Tensor, y: torch.Tensor, reduction: str = 'mean',
             threshold  point, below which phase congruency values get penalized.
         
     Returns:
-        FSIM: Index of similarity betwen two images. Usually in [0, 1] interval.
-            Can be bigger than 1 for predicted images with higher contrast than the original ones.
+        Index of similarity betwen two images. Usually in [0, 1] interval.
+        Can be bigger than 1 for predicted :math:`x` images with higher contrast than the original ones.
+
+    References:
+        L. Zhang, L. Zhang, X. Mou and D. Zhang, "FSIM: A Feature Similarity Index for Image Quality Assessment,"
+        IEEE Transactions on Image Processing, vol. 20, no. 8, pp. 2378-2386, Aug. 2011, doi: 10.1109/TIP.2011.2109730.
+        https://ieeexplore.ieee.org/document/5705575
+
     Note:
         This implementation is based on the original MATLAB code.
         https://www4.comp.polyu.edu.hk/~cslzhang/IQA/FSIM/FSIM.htm
         
     """
-    
-    _validate_input(input_tensors=(x, y), allow_5d=False, data_range=data_range)
-    x, y = _adjust_dimensions(input_tensors=(x, y))
-    
+    _validate_input([x, y], dim_range=(4, 4), data_range=(0, data_range))
+
     # Rescale to [0, 255] range, because all constant are calculated for this factor
-    x = x / data_range * 255
-    y = y / data_range * 255
-    
+    x = x / float(data_range) * 255
+    y = y / float(data_range) * 255
+
     # Apply average pooling
     kernel_size = max(1, round(min(x.shape[-2:]) / 256))
     x = torch.nn.functional.avg_pool2d(x, kernel_size)
     y = torch.nn.functional.avg_pool2d(y, kernel_size)
-        
+
     num_channels = x.size(1)
 
     # Convert RGB to YIQ color space https://en.wikipedia.org/wiki/YIQ
@@ -70,7 +74,7 @@ def fsim(x: torch.Tensor, y: torch.Tensor, reduction: str = 'mean',
 
         x_lum = x_yiq[:, : 1]
         y_lum = y_yiq[:, : 1]
-        
+
         x_i = x_yiq[:, 1:2]
         y_i = y_yiq[:, 1:2]
         x_q = x_yiq[:, 2:]
@@ -91,21 +95,21 @@ def fsim(x: torch.Tensor, y: torch.Tensor, reduction: str = 'mean',
         min_length=min_length, mult=mult, sigma_f=sigma_f,
         delta_theta=delta_theta, k=k
     )
-    
+
     # Gradient maps
     kernels = torch.stack([scharr_filter(), scharr_filter().transpose(-1, -2)])
     grad_map_x = gradient_map(x_lum, kernels)
     grad_map_y = gradient_map(y_lum, kernels)
-    
+
     # Constants from the paper
     T1, T2, T3, T4, lmbda = 0.85, 160, 200, 200, 0.03
-    
+
     # Compute FSIM
     PC = similarity_map(pc_x, pc_y, T1)
     GM = similarity_map(grad_map_x, grad_map_y, T2)
     pc_max = torch.where(pc_x > pc_y, pc_x, pc_y)
     score = GM * PC * pc_max
-    
+
     if chromatic:
         assert num_channels == 3, "Chromatic component can be computed only for RGB images!"
         S_I = similarity_map(x_i, y_i, T3)
@@ -115,13 +119,8 @@ def fsim(x: torch.Tensor, y: torch.Tensor, reduction: str = 'mean',
         # score = score * torch.real((S_I * S_Q).to(torch.complex64) ** lmbda)
 
     result = score.sum(dim=[1, 2, 3]) / pc_max.sum(dim=[1, 2, 3])
-    
-    if reduction == 'none':
-        return result
 
-    return {'mean': result.mean,
-            'sum': result.sum
-            }[reduction](dim=0)
+    return _reduce(result, reduction)
 
 
 def _construct_filters(x: torch.Tensor, scales: int = 4, orientations: int = 4,
@@ -130,7 +129,7 @@ def _construct_filters(x: torch.Tensor, scales: int = 4, orientations: int = 4,
     """Creates a stack of filters used for computation of phase congruensy maps
     
     Args:
-        x: Tensor with shape (N, 1, H, W).
+        x: Tensor. Shape :math:`(N, 1, H, W)`.
         scales: Number of wavelets
         orientations: Number of filter orientations
         min_length: Wavelength of smallest scale filter
@@ -203,7 +202,7 @@ def _construct_filters(x: torch.Tensor, scales: int = 4, orientations: int = 4,
 
     spread = torch.stack(spread)
     log_gabor = torch.stack(log_gabor)
-    
+
     # Multiply, add batch dimension and transfer to correct device.
     filters = (spread.repeat_interleave(scales, dim=0) * log_gabor.repeat(orientations, 1, 1)).unsqueeze(0).to(x)
     return filters
@@ -215,8 +214,8 @@ def _phase_congruency(x: torch.Tensor, scales: int = 4, orientations: int = 4,
     r"""Compute Phase Congruence for a batch of greyscale images
 
     Args:
-        x: Tensor with shape (N, 1, H, W).
-        levels: Number of wavelet scales
+        x: Tensor. Shape :math:`(N, 1, H, W)`.
+        scales: Number of wavelet scales
         orientations: Number of filter orientations
         min_length: Wavelength of smallest scale filter
         mult: Scaling factor between successive filters
@@ -229,8 +228,9 @@ def _phase_congruency(x: torch.Tensor, scales: int = 4, orientations: int = 4,
         k: No of standard deviations of the noise energy beyond the mean
             at which we set the noise threshold point, below which phase
             congruency values get penalized.
+
     Returns:
-        PCmap: Tensor with shape BxHxW
+        Phase Congruency map with shape :math:`(N, H, W)`
 
     """
     EPS = torch.finfo(x.dtype).eps
@@ -238,15 +238,18 @@ def _phase_congruency(x: torch.Tensor, scales: int = 4, orientations: int = 4,
     N, _, H, W = x.shape
 
     # Fourier transform
-    imagefft = torch.rfft(x, 2, onesided=False)
-
     filters = _construct_filters(x, scales, orientations, min_length, mult, sigma_f, delta_theta, k)
-
-    # Note rescaling to match power record ifft2 of filter
-    filters_ifft = torch.ifft(torch.stack([filters, torch.zeros_like(filters)], dim=-1), 2)[..., 0] * math.sqrt(H * W)
-    
-    # Convolve image with even and odd filters
-    even_odd = torch.ifft(imagefft * filters.unsqueeze(-1), 2).view(N, orientations, scales, H, W, 2)
+    recommended_torch_version = '1.8.0'
+    if _version_tuple(torch.__version__) >= _version_tuple(recommended_torch_version):
+        imagefft = torch.fft.fft2(x)
+        filters_ifft = torch.fft.ifft2(filters)
+        filters_ifft = filters_ifft.real * math.sqrt(H * W)
+        even_odd = torch.view_as_real(torch.fft.ifft2(imagefft * filters)).view(N, orientations, scales, H, W, 2)
+    else:
+        imagefft = torch.rfft(x, 2, onesided=False)
+        filters_ifft = torch.ifft(torch.stack([filters, torch.zeros_like(filters)], dim=-1), 2)[..., 0]
+        filters_ifft *= math.sqrt(H * W)
+        even_odd = torch.ifft(imagefft * filters.unsqueeze(-1), 2).view(N, orientations, scales, H, W, 2)
 
     # Amplitude of even & odd filter response. An = sqrt(real^2 + imag^2)
     an = torch.sqrt(torch.sum(even_odd ** 2, dim=-1))
@@ -258,10 +261,10 @@ def _phase_congruency(x: torch.Tensor, scales: int = 4, orientations: int = 4,
 
     # Sum of even filter convolution results.
     sum_e = even_odd[..., 0].sum(dim=2, keepdims=True)
-    
+
     # Sum of odd filter convolution results.
     sum_o = even_odd[..., 1].sum(dim=2, keepdims=True)
-    
+
     # Get weighted mean filter response vector, this gives the weighted mean phase angle.
     x_energy = torch.sqrt(sum_e ** 2 + sum_o ** 2) + EPS
 
@@ -278,7 +281,7 @@ def _phase_congruency(x: torch.Tensor, scales: int = 4, orientations: int = 4,
     odd = even_odd[..., 1]
 
     energy = (even * mean_e + odd * mean_o - torch.abs(even * mean_o - odd * mean_e)).sum(dim=2, keepdim=True)
-    
+
     # Compensate for noise
     # We estimate the noise power from the energy squared response at the
     # smallest scale.  If the noise is Gaussian the energy squared will have a
@@ -286,7 +289,7 @@ def _phase_congruency(x: torch.Tensor, scales: int = 4, orientations: int = 4,
     # as this is a robust statistic.  From this we estimate the mean.
     # The estimate of noise power is obtained by dividing the mean squared
     # energy value by the mean squared filter value
-    
+
     abs_eo = torch.sqrt(torch.sum(even_odd[:, :, :1, ...] ** 2, dim=-1)).reshape(N, orientations, 1, 1, H * W)
     median_e2n = torch.median(abs_eo ** 2, dim=-1, keepdim=True).values
 
@@ -294,17 +297,17 @@ def _phase_congruency(x: torch.Tensor, scales: int = 4, orientations: int = 4,
 
     # Estimate of noise power.
     noise_power = mean_e2n / em_n
-    
+
     # Now estimate the total energy^2 due to noise
     # Estimate for sum(An^2) + sum(Ai.*Aj.*(cphi.*cphj + sphi.*sphj))
     filters_ifft = filters_ifft.view(1, orientations, scales, H, W)
-    
+
     sum_an2 = torch.sum(filters_ifft ** 2, dim=-3, keepdim=True)
-    
+
     sum_ai_aj = torch.zeros(N, orientations, 1, H, W).to(x)
     for s in range(scales - 1):
         sum_ai_aj = sum_ai_aj + (filters_ifft[:, :, s: s + 1] * filters_ifft[:, :, s + 1:]).sum(dim=-3, keepdim=True)
-            
+
     sum_an2 = torch.sum(sum_an2, dim=[-1, -2], keepdim=True)
     sum_ai_aj = torch.sum(sum_ai_aj, dim=[-1, -2], keepdim=True)
 
@@ -341,6 +344,7 @@ def _phase_congruency(x: torch.Tensor, scales: int = 4, orientations: int = 4,
 def _lowpassfilter(size: Tuple[int, int], cutoff: float, n: int) -> torch.Tensor:
     r"""
     Constructs a low-pass Butterworth filter.
+
     Args:
         size: Tuple with heigth and width of filter to construct
         cutoff: Cutoff frequency of the filter in (0, 0.5()
@@ -358,7 +362,7 @@ def _lowpassfilter(size: Tuple[int, int], cutoff: float, n: int) -> torch.Tensor
     assert n > 1 and int(n) == n, "n must be an integer >= 1"
 
     grid_x, grid_y = get_meshgrid(size)
-    
+
     # A matrix with every pixel = radius relative to centre.
     radius = torch.sqrt(grid_x ** 2 + grid_y ** 2)
 
@@ -368,17 +372,14 @@ def _lowpassfilter(size: Tuple[int, int], cutoff: float, n: int) -> torch.Tensor
 class FSIMLoss(_Loss):
     r"""Creates a criterion that measures the FSIM or FSIMc for input :math:`x` and target :math:`y`.
 
-    In order to be considered as a loss, value `1 - clip(FSIM, min=0, max=1)` is returned. If you need FSIM value,
+    In order to be considered as a loss, value ``1 - clip(FSIM, min=0, max=1)`` is returned. If you need FSIM value,
     use function `fsim` instead.
+    Supports greyscale and colour images with RGB channel order.
 
     Args:
-        reduction: Specifies the reduction to apply to the output:
-            ``'none'`` | ``'mean'`` | ``'sum'``. ``'none'``: no reduction will be applied,
-            ``'mean'``: the sum of the output will be divided by the number of
-            elements in the output, ``'sum'``: the output will be summed. Default: ``'mean'``
-        data_range: The difference between the maximum and minimum of the pixel value,
-            i.e., if for image x it holds min(x) = 0 and max(x) = 1, then data_range = 1.
-            The pixel value interval of both input and output should remain the same.
+        reduction: Specifies the reduction type:
+            ``'none'`` | ``'mean'`` | ``'sum'``. Default:``'mean'``
+        data_range: Maximum value range of images (usually 1.0 or 255).
         chromatic: Flag to compute FSIMc, which also takes into account chromatic components
         scales: Number of wavelets used for computation of phase congruensy maps
         orientations: Number of filter orientations used for computation of phase congruensy maps
@@ -391,22 +392,19 @@ class FSIMLoss(_Loss):
         k: No of standard deviations of the noise energy beyond the mean at which we set the noise
             threshold  point, below which phase congruency values get penalized.
 
-    Shape:
-        - Input: Required to be 2D (H, W), 3D (C, H, W) or 4D (N, C, H, W). RGB channel order for colour images.
-        - Target: Required to be 2D (H, W), 3D (C, H, W) or 4D (N, C, H, W). RGB channel order for colour images.
-
-    Examples::
-
+    Examples:
         >>> loss = FSIMLoss()
-        >>> prediction = torch.rand(3, 3, 256, 256, requires_grad=True)
-        >>> target = torch.rand(3, 3, 256, 256)
-        >>> output = loss(prediction, target)
+        >>> x = torch.rand(3, 3, 256, 256, requires_grad=True)
+        >>> y = torch.rand(3, 3, 256, 256)
+        >>> output = loss(x, y)
         >>> output.backward()
 
     References:
-        .. [1] Anish Mittal et al. "No-Reference Image Quality Assessment in the Spatial Domain",
-        https://live.ece.utexas.edu/publications/2012/TIP%20BRISQUE.pdf
-        """
+        L. Zhang, L. Zhang, X. Mou and D. Zhang, "FSIM: A Feature Similarity Index for Image Quality Assessment,"
+        IEEE Transactions on Image Processing, vol. 20, no. 8, pp. 2378-2386, Aug. 2011, doi: 10.1109/TIP.2011.2109730.
+        https://ieeexplore.ieee.org/document/5705575
+    """
+
     def __init__(self, reduction: str = 'mean', data_range: Union[int, float] = 1., chromatic: bool = True,
                  scales: int = 4, orientations: int = 4, min_length: int = 6, mult: int = 2,
                  sigma_f: float = 0.55, delta_theta: float = 1.2, k: float = 2.0) -> None:
@@ -429,16 +427,18 @@ class FSIMLoss(_Loss):
             k=k,
         )
 
-    def forward(self, prediction: torch.Tensor, target: torch.Tensor) -> torch.Tensor:
+    def forward(self, x: torch.Tensor, y: torch.Tensor) -> torch.Tensor:
         r"""Computation of FSIM as a loss function.
+
         Args:
-            prediction: Tensor of prediction of the network.
-            target: Reference tensor.
+            x: An input tensor. Shape :math:`(N, C, H, W)`.
+            y: A target tensor. Shape :math:`(N, C, H, W)`.
+
         Returns:
-            Value of FSIM loss to be minimized. 0 <= FSIM <= 1.
+            Value of FSIM loss to be minimized in [0, 1] range.
         """
         # All checks are done inside fsim function
-        score = self.fsim(prediction, target)
+        score = self.fsim(x, y)
 
         # Make sure value to be in [0, 1] range and convert to loss
         return 1 - torch.clamp(score, 0, 1)
