@@ -6,9 +6,9 @@ from typing import Tuple
 
 # ================== Test function: `vsi` ==================
 def test_vsi_to_be_one_for_identical_inputs(input_tensors: Tuple[torch.Tensor, torch.Tensor], device: str) -> None:
-    prediction, _ = input_tensors
-    index = vsi(prediction.to(device), prediction.to(device), data_range=1., reduction='none')
-    index_255 = vsi(prediction.to(device) * 255, prediction.to(device) * 255, data_range=255, reduction='none')
+    x, _ = input_tensors
+    index = vsi(x.to(device), x.to(device), data_range=1., reduction='none')
+    index_255 = vsi(x.to(device) * 255, x.to(device) * 255, data_range=255, reduction='none')
     assert torch.allclose(index, torch.ones_like(index, device=device)), \
         f'Expected index to be equal 1, got {index}'
     assert torch.allclose(index_255, torch.ones_like(index_255, device=device)), \
@@ -16,9 +16,9 @@ def test_vsi_to_be_one_for_identical_inputs(input_tensors: Tuple[torch.Tensor, t
 
 
 def test_vsi_symmetry(input_tensors: Tuple[torch.Tensor, torch.Tensor], device: str) -> None:
-    prediction, target = input_tensors
-    result = vsi(prediction.to(device), target.to(device), data_range=1., reduction='none')
-    result_sym = vsi(target.to(device), prediction.to(device), data_range=1., reduction='none')
+    x, y = input_tensors
+    result = vsi(x.to(device), y.to(device), data_range=1., reduction='none')
+    result_sym = vsi(y.to(device), x.to(device), data_range=1., reduction='none')
     assert torch.allclose(result_sym, result), f'Expected the same results, got {result} and {result_sym}'
 
 
@@ -35,9 +35,9 @@ def test_vsi_zeros_ones_inputs(device: str) -> None:
 
 
 def test_vsi_compare_with_matlab(device: str) -> None:
-    prediction = torch.tensor(imread('tests/assets/I01.BMP')).permute(2, 0, 1)
-    target = torch.tensor(imread('tests/assets/i01_01_5.bmp')).permute(2, 0, 1)
-    predicted_score = vsi(prediction.to(device), target.to(device), data_range=255, reduction='none')
+    x = torch.tensor(imread('tests/assets/I01.BMP')).permute(2, 0, 1)[None, ...]
+    y = torch.tensor(imread('tests/assets/i01_01_5.bmp')).permute(2, 0, 1)[None, ...]
+    predicted_score = vsi(x.to(device), y.to(device), data_range=255, reduction='none')
     target_score = torch.tensor([0.96405]).to(predicted_score)
     assert torch.allclose(predicted_score, target_score), f'Expected result similar to MATLAB,' \
                                                           f'got diff{predicted_score - target_score}'
@@ -45,18 +45,18 @@ def test_vsi_compare_with_matlab(device: str) -> None:
 
 # ================== Test class: `VSILoss` =================
 def test_vsi_loss(input_tensors: Tuple[torch.Tensor, torch.Tensor], device: str) -> None:
-    prediction, target = input_tensors
-    prediction.requires_grad_()
-    loss = VSILoss(data_range=1.)(prediction.to(device), target.to(device))
+    x, y = input_tensors
+    x.requires_grad_()
+    loss = VSILoss(data_range=1.)(x.to(device), y.to(device))
     loss.backward()
-    assert torch.isfinite(prediction.grad).all(), \
-        f'Expected finite gradient values after back propagation, got {prediction.grad}'
+    assert torch.isfinite(x.grad).all(), \
+        f'Expected finite gradient values after back propagation, got {x.grad}'
 
 
 def test_vsi_loss_zero_for_equal_input(input_tensors: Tuple[torch.Tensor, torch.Tensor], device: str) -> None:
-    prediction, _ = input_tensors
-    target = prediction.clone()
-    prediction.requires_grad_()
-    loss = VSILoss(data_range=1.)(prediction.to(device), target.to(device))
+    x, _ = input_tensors
+    y = x.clone()
+    x.requires_grad_()
+    loss = VSILoss(data_range=1.)(x.to(device), y.to(device))
     assert torch.isclose(loss, torch.zeros_like(loss)), \
         f'Expected loss equals zero for identical inputs, got {loss}'
